import { defineNuxtModule, useLogger, createResolver, addImportsDir, addTemplate, resolveModule } from '@nuxt/kit';
import defu from 'defu';
import { joinURL } from 'ufo';

const PACKAGE_NAME = "nuxt-auth";
const defaults = {
  isEnabled: true,
  origin: void 0,
  basePath: "/api/auth"
};
const module = defineNuxtModule({
  meta: {
    name: PACKAGE_NAME,
    configKey: "auth"
  },
  defaults,
  setup(moduleOptions, nuxt) {
    const logger = useLogger(PACKAGE_NAME);
    if (!moduleOptions.isEnabled) {
      logger.info(`Skipping ${PACKAGE_NAME} setup, as module is disabled`);
      return;
    }
    logger.info("Setting up auth...");
    const isOriginSet = Boolean(moduleOptions.origin);
    const usedOrigin = moduleOptions.origin ?? "http://localhost:3000";
    const options = defu(moduleOptions, {
      ...defaults,
      basePath: defaults.basePath,
      origin: usedOrigin
    });
    const url = joinURL(usedOrigin, options.basePath);
    logger.info(`Using \`${url}\` as the auth API location, make sure the \`[...].ts\` file with the \`export default NuxtAuthHandler({ ... })\` is added there. Use the \`nuxt.config.ts\` \`auth.origin\` and \`auth.basePath\` config keys to change the API location`);
    if (process.env.NODE_ENV === "production") {
      logger.info("When building for production ensure to (1) set the application origin using `auth.origin` inside your `nuxt.config.ts` and (2) set the secret inside the `NuxtAuthHandler({ secret: ... })`");
    }
    nuxt.options.runtimeConfig = nuxt.options.runtimeConfig || { public: {} };
    nuxt.options.runtimeConfig.auth = defu(nuxt.options.runtimeConfig.auth, {
      ...options,
      url,
      isOriginSet
    });
    nuxt.options.runtimeConfig.public.auth = defu(nuxt.options.runtimeConfig.public.auth, {
      url
    });
    const { resolve } = createResolver(import.meta.url);
    const resolveRuntimeModule = (path) => resolveModule(path, { paths: resolve("./runtime") });
    const composables = resolve("./runtime/composables");
    addImportsDir(composables);
    nuxt.hook("nitro:config", (nitroConfig) => {
      nitroConfig.alias = nitroConfig.alias || {};
      nitroConfig.externals = defu(typeof nitroConfig.externals === "object" ? nitroConfig.externals : {}, {
        inline: [resolve("./runtime")]
      });
      nitroConfig.alias["#auth"] = resolveRuntimeModule("./server/services");
    });
    addTemplate({
      filename: "types/auth.d.ts",
      getContents: () => [
        "declare module '#auth' {",
        `  const getServerSession: typeof import('${resolve("./runtime/server/services")}').getServerSession`,
        `  const getToken: typeof import('${resolve("./runtime/server/services")}').getToken`,
        `  const NuxtAuthHandler: typeof import('${resolve("./runtime/server/services")}').NuxtAuthHandler`,
        "}"
      ].join("\n")
    });
    nuxt.hook("prepare:types", (options2) => {
      options2.references.push({ path: resolve(nuxt.options.buildDir, "types/auth.d.ts") });
    });
    logger.success("Auth module setup done");
  }
});

export { module as default };
