import defu from "defu";
import { joinURL, parseURL } from "ufo";
import { callWithNuxt } from "#app";
import { createError, useState, useRuntimeConfig, useRequestHeaders, navigateTo, useRequestEvent, useNuxtApp } from "#imports";
const _getBasePath = () => parseURL(useRuntimeConfig().public.auth.url).pathname;
const joinPathToBase = (path) => joinURL(_getBasePath(), path);
const getUniversalRequestUrl = () => {
  const event = useRequestEvent();
  if (event) {
    return event.req.url || "/";
  }
  if (window) {
    return window.location.href;
  }
  throw new Error("Unexpected runtime error, request must either run on client or on server!");
};
const universalRedirect = (href, { external } = { external: true }) => {
  if (process.client) {
    window.location.href = href;
    if (href.includes("#")) {
      window.location.reload();
    }
  } else {
    return navigateTo(href, { external });
  }
};
const _fetch = async (path, { body, params, method, headers, onResponse, onRequest, onRequestError, onResponseError } = { params: {}, headers: {}, method: "GET" }) => {
  try {
    const res = await $fetch(joinPathToBase(path), {
      method,
      params,
      headers,
      body,
      onResponse,
      onRequest,
      onRequestError,
      onResponseError
    });
    return res;
  } catch (error) {
    console.error("Error in useSession data fetching: Have you added the authentication handler server-endpoint `[...].ts`? Have you added the authentication handler in a non-default location (default is `~/server/api/auth/[...].ts`) and not updated the module-setting `auth.basePath`? Error is:");
    console.error(error);
    throw new Error("Runtime error, checkout the console logs to debug, open an issue at https://github.com/sidebase/nuxt-auth/issues/new/choose if you continue to have this problem");
  }
};
export default async (initialGetSessionOptions = {}) => {
  const data = useState("session:data", () => void 0);
  const status = useState("session:status", () => "unauthenticated");
  const signIn = async (provider, options, authorizationParams) => {
    const configuredProviders = await getProviders();
    if (!configuredProviders) {
      return universalRedirect(joinPathToBase("error"));
    }
    const { callbackUrl = getUniversalRequestUrl(), redirect = true } = options ?? {};
    const hrefSignInAllProviderPage = `${joinPathToBase("signin")}?${new URLSearchParams({ callbackUrl })}`;
    if (!provider) {
      return universalRedirect(hrefSignInAllProviderPage);
    }
    const selectedProvider = configuredProviders[provider];
    if (!selectedProvider) {
      return universalRedirect(hrefSignInAllProviderPage);
    }
    const isCredentials = selectedProvider.type === "credentials";
    const isEmail = selectedProvider.type === "email";
    const isSupportingReturn = isCredentials || isEmail;
    let action = "signin";
    if (isCredentials) {
      action = "callback";
    }
    const csrfTokenResult = await getCsrfToken();
    const csrfToken = csrfTokenResult.csrfToken;
    const data2 = await _fetch(`${action}/${provider}`, {
      method: "post",
      headers: {
        "Content-Type": "application/x-www-form-urlencoded"
      },
      params: authorizationParams,
      body: new URLSearchParams({
        ...options,
        csrfToken,
        callbackUrl,
        json: true
      })
    });
    if (redirect || !isSupportingReturn) {
      const href = data2.url ?? callbackUrl;
      return universalRedirect(href);
    }
    const error = new URL(data2.url).searchParams.get("error");
    return {
      error,
      status: 200,
      ok: true,
      url: error ? null : data2.url
    };
  };
  const signOut = async (options) => {
    const { callbackUrl = getUniversalRequestUrl(), redirect = true } = options ?? {};
    const csrfTokenResult = await getCsrfToken();
    const csrfToken = csrfTokenResult.csrfToken;
    if (!csrfToken) {
      throw createError({ statusCode: 400, statusMessage: "Could not fetch CSRF Token for signing out" });
    }
    const onRequest = ({ options: options2 }) => {
      options2.body = new URLSearchParams({
        csrfToken,
        callbackUrl: callbackUrl || getUniversalRequestUrl(),
        json: "true"
      });
    };
    const signoutData = await _fetch("signout", {
      method: "POST",
      headers: {
        "Content-Type": "application/x-www-form-urlencoded"
      },
      onRequest
    });
    status.value = "unauthenticated";
    data.value = void 0;
    if (redirect) {
      const url = signoutData.url ?? callbackUrl;
      return universalRedirect(url);
    }
    return signoutData;
  };
  const getCsrfToken = () => {
    let headers = {};
    const { cookie } = useRequestHeaders(["cookie"]);
    if (cookie) {
      headers = { cookie };
    }
    return _fetch("csrf", { headers });
  };
  const getProviders = () => _fetch("providers");
  const getSession = async (getSessionOptions) => {
    const nuxt = useNuxtApp();
    const { required, callbackUrl, onUnauthenticated } = defu(getSessionOptions || {}, {
      required: true,
      callbackUrl: void 0,
      onUnauthenticated: () => universalRedirect(joinPathToBase(`signin?${new URLSearchParams({ callbackUrl: getSessionOptions?.callbackUrl || "/" })}`), { external: true })
    });
    const onRequest = ({ options }) => {
      status.value = "loading";
      options.params = {
        ...options.params || {},
        callbackUrl: callbackUrl || getUniversalRequestUrl()
      };
    };
    const onResponse = ({ response }) => {
      const sessionData = response._data;
      if (!sessionData || Object.keys(sessionData).length === 0) {
        status.value = "unauthenticated";
        data.value = null;
      } else {
        status.value = "authenticated";
        data.value = sessionData;
      }
      return sessionData;
    };
    const onError = () => {
      status.value = "unauthenticated";
    };
    let headers = {};
    const { cookie } = useRequestHeaders(["cookie"]);
    if (cookie) {
      headers = { cookie };
    }
    const result = await _fetch("session", {
      onResponse,
      onRequest,
      onRequestError: onError,
      onResponseError: onError,
      headers
    });
    if (required && status.value === "unauthenticated") {
      const result2 = await callWithNuxt(nuxt, onUnauthenticated, []);
      return result2;
    }
    return result;
  };
  const initialGetSessionOptionsWithDefaults = defu(initialGetSessionOptions, {
    required: true,
    onUnauthenticated: void 0,
    callbackUrl: void 0
  });
  await getSession(initialGetSessionOptionsWithDefaults);
  const actions = {
    getSession,
    getCsrfToken,
    getProviders,
    signIn,
    signOut
  };
  const getters = {
    status,
    data
  };
  return {
    ...actions,
    ...getters
  };
};
